@@ -1,9 +1,9 @@
 /* $XFree86$ */
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/drivers/sis/sis_driver.h,v 1.8 2004/08/12 12:59:25 twini Exp $ */
+/* $XdotOrg$ */
 /*
  * Global data and definitions
  *
- * Copyright (C) 2001-2004 by Thomas Winischhofer, Vienna, Austria
+ * Copyright (C) 2001-2005 by Thomas Winischhofer, Vienna, Austria
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,22 +34,25 @@
 /* VESA */
 /*     The following is included because there are BIOSes out there that
  *     report incomplete mode lists. These are 630 BIOS versions <2.01.2x
- *
  *     -) VBE 3.0 on SiS300 and 315 series do not support 24 fpp modes
  *     -) Only SiS315 series support 1920x1440x32
  */
-				             /*     8      16    (24)    32   */
-static const UShort  VESAModeIndex_320x200[]   = {0x138, 0x10e, 0x000, 0x000};
-static const UShort  VESAModeIndex_320x240[]   = {0x132, 0x135, 0x000, 0x000};
-static const UShort  VESAModeIndex_400x300[]   = {0x133, 0x136, 0x000, 0x000};
-static const UShort  VESAModeIndex_512x384[]   = {0x134, 0x137, 0x000, 0x000};
-static const UShort  VESAModeIndex_640x400[]   = {0x100, 0x139, 0x000, 0x000};
-static const UShort  VESAModeIndex_640x480[]   = {0x101, 0x111, 0x000, 0x13a};
-static const UShort  VESAModeIndex_800x600[]   = {0x103, 0x114, 0x000, 0x13b};
-static const UShort  VESAModeIndex_1024x768[]  = {0x105, 0x117, 0x000, 0x13c};
-static const UShort  VESAModeIndex_1280x1024[] = {0x107, 0x11a, 0x000, 0x13d};
-static const UShort  VESAModeIndex_1600x1200[] = {0x130, 0x131, 0x000, 0x13e};
-static const UShort  VESAModeIndex_1920x1440[] = {0x13f, 0x140, 0x000, 0x141};
+
+static const UShort VESAModeIndices[] = {
+   /*   x    y     8      16    (24)    32   */
+       320, 200, 0x138, 0x10e, 0x000, 0x000,
+       320, 240, 0x132, 0x135, 0x000, 0x000,
+       400, 300, 0x133, 0x136, 0x000, 0x000,
+       512, 384, 0x134, 0x137, 0x000, 0x000,
+       640, 400, 0x100, 0x139, 0x000, 0x000,
+       640, 480, 0x101, 0x111, 0x000, 0x13a,
+       800, 600, 0x103, 0x114, 0x000, 0x13b,
+      1024, 768, 0x105, 0x117, 0x000, 0x13c,
+      1280,1024, 0x107, 0x11a, 0x000, 0x13d,
+      1600,1200, 0x130, 0x131, 0x000, 0x13e,
+      1920,1440, 0x13f, 0x140, 0x000, 0x141,
+      9999,9999, 0,     0,     0,     0
+};
 
 /* For calculating refresh rate index (CR33) */
 static const struct _sis_vrate {
@@ -57,22 +60,22 @@ static const struct _sis_vrate {
     CARD16 xres;
     CARD16 yres;
     CARD16 refresh;
-    BOOLEAN SiS730valid32bpp;
+    Bool SiS730valid32bpp;
 } sisx_vrate[] = {
-	{1,  320,  200,  70,  TRUE},
+	{1,  320,  200,  60,  TRUE}, {1,  320,  200,  70,  TRUE},
 	{1,  320,  240,  60,  TRUE},
 	{1,  400,  300,  60,  TRUE},
         {1,  512,  384,  60,  TRUE},
-	{1,  640,  400,  72,  TRUE},
-	{1,  640,  480,  60,  TRUE}, {2,  640,  480,  72,  TRUE}, {3,  640,  480,  75,  TRUE}, 
-	{4,  640,  480,  85,  TRUE}, {5,  640,  480, 100,  TRUE}, {6,  640,  480, 120,  TRUE}, 
+	{1,  640,  400,  60,  TRUE}, {1,  640,  400,  72,  TRUE},
+	{1,  640,  480,  60,  TRUE}, {2,  640,  480,  72,  TRUE}, {3,  640,  480,  75,  TRUE},
+	{4,  640,  480,  85,  TRUE}, {5,  640,  480, 100,  TRUE}, {6,  640,  480, 120,  TRUE},
 	{7,  640,  480, 160, FALSE}, {8,  640,  480, 200, FALSE},
 	{1,  720,  480,  60,  TRUE},
-	{1,  720,  576,  58,  TRUE},
-	{1,  768,  576,  58,  TRUE},
+	{1,  720,  576,  60,  TRUE},
+	{1,  768,  576,  60,  TRUE},
 	{1,  800,  480,  60,  TRUE}, {2,  800,  480,  75,  TRUE}, {3,  800,  480,  85,  TRUE},
-	{1,  800,  600,  56,  TRUE}, {2,  800,  600,  60,  TRUE}, {3,  800,  600,  72,  TRUE}, 
-	{4,  800,  600,  75,  TRUE}, {5,  800,  600,  85,  TRUE}, {6,  800,  600, 105,  TRUE}, 
+	{1,  800,  600,  56,  TRUE}, {2,  800,  600,  60,  TRUE}, {3,  800,  600,  72,  TRUE},
+	{4,  800,  600,  75,  TRUE}, {5,  800,  600,  85,  TRUE}, {6,  800,  600, 105,  TRUE},
 	{7,  800,  600, 120, FALSE}, {8,  800,  600, 160, FALSE},
 	{1,  848,  480,  39,  TRUE}, {2,  848,  480,  60,  TRUE},
 	{1,  856,  480,  39,  TRUE}, {2,  856,  480,  60,  TRUE},
@@ -84,10 +87,11 @@ static const struct _sis_vrate {
 	{4, 1024,  768,  75, FALSE}, {5, 1024,  768,  85,  TRUE}, {6, 1024,  768, 100,  TRUE},
 	{7, 1024,  768, 120,  TRUE},
 	{1, 1152,  768,  60,  TRUE},
-	{1, 1152,  864,  75,  TRUE}, {2, 1152,  864,  84, FALSE},
+	{1, 1152,  864,  60,  TRUE}, {2, 1152,  864,  75,  TRUE}, {3, 1152,  864,  84, FALSE},
 	{1, 1280,  720,  60,  TRUE}, {2, 1280,  720,  75, FALSE}, {3, 1280,  720,  85,  TRUE},
-	{1, 1280,  768,  60,  TRUE},
-	{1, 1280,  800,  60,  TRUE},
+	{1, 1280,  768,  60,  TRUE}, {2, 1280,  768,  75,  TRUE}, {3, 1280,  768,  85,  TRUE},
+	{1, 1280,  800,  60,  TRUE}, {2, 1280,  800,  75,  TRUE}, {3, 1280,  800,  85,  TRUE},
+	{1, 1280,  854,  60,  TRUE}, {2, 1280,  854,  75,  TRUE}, {3, 1280,  854,  85,  TRUE},
 	{1, 1280,  960,  60,  TRUE}, {2, 1280,  960,  85,  TRUE},
 	{1, 1280, 1024,  43, FALSE}, {2, 1280, 1024,  60,  TRUE}, {3, 1280, 1024,  75, FALSE},
 	{4, 1280, 1024,  85,  TRUE},
@@ -100,7 +104,7 @@ static const struct _sis_vrate {
 	{1, 1920, 1080,  30,  TRUE},
 	{1, 1920, 1440,  60,  TRUE}, {2, 1920, 1440,  65,  TRUE}, {3, 1920, 1440,  70,  TRUE},
 	{4, 1920, 1440,  75,  TRUE}, {5, 1920, 1440,  85,  TRUE}, {6, 1920, 1440, 100,  TRUE},
-	{1, 2048, 1536,  60,  TRUE}, {2, 2048, 1536,  65,  TRUE}, {3, 2048, 1536,  70,  TRUE}, 
+	{1, 2048, 1536,  60,  TRUE}, {2, 2048, 1536,  65,  TRUE}, {3, 2048, 1536,  70,  TRUE},
 	{4, 2048, 1536,  75,  TRUE}, {5, 2048, 1536,  85,  TRUE},
 	{0,    0,    0,   0, FALSE}
 };
@@ -128,7 +132,7 @@ static const chswtable mychswtable[] = {
 
 /*     These machines require special timing/handling
  */
-const customttable mycustomttable[] = {
+const customttable SiS_customttable[] = {
         { SIS_630, "2.00.07", "09/27/2002-13:38:25",
 	  0x3240A8,
 	  { 0x220, 0x227, 0x228, 0x229, 0x0ee },
@@ -241,6 +245,15 @@ const customttable mycustomttable[] = {
 	  0x1043, 0x1612,
 	  "Asus", "A2H (V2)", CUT_ASUSA2H_2, "ASUS_A2H_2"
 	},
+#if 0
+	{ SIS_550, "1.02.0z", "",
+	  0x317f37,	/* 320x240 LCD panel */
+	  { 0, 0, 0, 0, 0 },
+	  { 0, 0, 0, 0, 0 },
+	  0, 0,
+	  "AAEON", "AOP-8060", CUT_AOP8060, "AAEON_AOP_8060"
+	},
+#endif
 	{ 4321, "", "",			/* never autodetected */
 	  0,
 	  { 0, 0, 0, 0, 0 },
@@ -248,6 +261,13 @@ const customttable mycustomttable[] = {
 	  0, 0,
 	  "Generic", "LVDS/Parallel 848x480", CUT_PANEL848, "PANEL848x480"
 	},
+	{ 4322, "", "",			/* never autodetected */
+	  0,
+	  { 0, 0, 0, 0, 0 },
+	  { 0, 0, 0, 0, 0 },
+	  0, 0,
+	  "Generic", "LVDS/Parallel 856x480", CUT_PANEL856, "PANEL856x480"
+	},
 	{ 0, "", "",
 	  0,
 	  { 0, 0, 0, 0 },
@@ -655,7 +675,7 @@ static DisplayModeRec SiS6326SIS1600x1200_60Mode = {
 /*     TV filters for SiS video bridges
  */
 static const struct _SiSTVFilter301 {
-	unsigned char filter[7][4];
+	UChar filter[7][4];
 } SiSTVFilter301[] = {
 	{{ {0x00,0xE0,0x10,0x60},   /* NTSCFilter - 320 */
 	   {0x00,0xEE,0x10,0x44},
@@ -716,7 +736,7 @@ static const struct _SiSTVFilter301 {
 };
 
 static const struct _SiSTVFilter301B {
-	unsigned char filter[7][7];
+	UChar filter[7][7];
 } SiSTVFilter301B[] = {
 	{{ {0x01,0x02,0xfb,0xf8,0x06,0x27,0x3a},   /* NTSC - 640 */
 	   {0x01,0x02,0xfe,0xf7,0x03,0x27,0x3c},
@@ -779,16 +799,16 @@ static const struct _SiSTVFilter301B {
 /*     TV scaling data for SiS video bridges
  */
 typedef struct _SiSTVVScale {
-        unsigned short ScaleVDE;
+        UShort ScaleVDE;
 	int sindex;
-	unsigned short RealVDE;
+	UShort RealVDE;
 #if 0
-	unsigned short HT, HRS, HRE, VT, VRS, VRE;
-	unsigned short NFF, HCFACT, HCMAX, VBHT, VBVT, VBHRS;
-	unsigned short HT300, HRS300, HRE300, VT300, VRS300, VRE300;
-	unsigned short NFF300, HCFACT300, HCMAX300, VBHT300, VBVT300, VBHRS300;
+	UShort HT, HRS, HRE, VT, VRS, VRE;
+	UShort NFF, HCFACT, HCMAX, VBHT, VBVT, VBHRS;
+	UShort HT300, HRS300, HRE300, VT300, VRS300, VRE300;
+	UShort NFF300, HCFACT300, HCMAX300, VBHT300, VBVT300, VBHRS300;
 #endif
-	unsigned short reg[24];
+	UShort reg[24];
 } MySiSTVVScale, *MySiSTVVScalePtr;
 
 static const MySiSTVVScale SiSTVVScale[] = {
@@ -1031,14 +1051,14 @@ static const MySiSTVVScale SiSTVVScale[] = {
       },
 };
 
-unsigned const char SiSScalingP1Regs[] = {
+static unsigned const char SiSScalingP1Regs[] = {
 	0x08,0x09,0x0b,0x0c,0x0d,0x0e,0x10,0x11,0x12
 };
-unsigned const char SiSScalingP4Regs[] = {
+static unsigned const char SiSScalingP4Regs[] = {
 	0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b
 };
 
-static const unsigned char SiS301CScaling[] = {
+static const UChar SiS301CScaling[] = {
 
     /* NTSC/PAL-M/525ip 640x480 */
 
@@ -1263,6 +1283,78 @@ static const unsigned char SiS301CScaling[] = {
     0x7D,0x0C,0x16,0x01,0x7D,0x0A,0x17,0x02,0x7D,0x09,0x17,0x03,0x7D,0x07,0x17,0x05
 };
 
+/* For communication with the SiS Linux framebuffer driver (sisfb) */
+
+/* ioctl for identifying and giving some info (esp. memory heap start) */
+#define SISFB_GET_INFO_SIZE	0x8004f300
+#define SISFB_GET_INFO		0x8000f301  /* Must be patched with result from ..._SIZE at D[29:16] */
+/* deprecated ioctl number (for older versions of sisfb) */
+#define SISFB_GET_INFO_OLD	0x80046ef8
+
+/* ioctls for tv parameters (position) */
+#define SISFB_SET_TVPOSOFFSET	0x4004f304
+
+/* lock sisfb from register access */
+#define SISFB_SET_LOCK		0x4004f306
+
+/* Magic value for USB device */
+#ifndef SISFB_USB_MAGIC
+#define SISFB_USB_MAGIC		0x55aa2011
+#endif
+
+/* Structure argument for SISFB_GET_INFO ioctl  */
+typedef struct _SISFB_INFO sisfb_info, *psisfb_info;
+
+struct _SISFB_INFO {
+	CARD32 	sisfb_id;		/* for identifying sisfb */
+#ifndef SISFB_ID
+#define SISFB_ID	  0x53495346	/* Identify myself with 'SISF' */
+#endif
+ 	CARD32 	chip_id;		/* PCI ID of detected chip */
+	CARD32	memory;			/* video memory in KB which sisfb manages */
+	CARD32	heapstart;		/* heap start (= sisfb "mem" argument) in KB */
+	CARD8 	fbvidmode;		/* current sisfb mode */
+
+	CARD8 	sisfb_version;
+	CARD8	sisfb_revision;
+	CARD8 	sisfb_patchlevel;
+
+	CARD8 	sisfb_caps;		/* sisfb's capabilities */
+
+	CARD32 	sisfb_tqlen;		/* turbo queue length (in KB) */
+
+	CARD32 	sisfb_pcibus;		/* The card's PCI bus ID. For USB, bus = SISFB_USB_MAGIC */
+	CARD32 	sisfb_pcislot;		/* alias usbbus */
+	CARD32 	sisfb_pcifunc;		/* alias usbdev */
+
+	CARD8 	sisfb_lcdpdc;
+
+	CARD8	sisfb_lcda;
+
+	CARD32	sisfb_vbflags;
+	CARD32	sisfb_currentvbflags;
+
+	CARD32 	sisfb_scalelcd;
+	CARD32 	sisfb_specialtiming;
+
+	CARD8 	sisfb_haveemi;
+	CARD8 	sisfb_emi30,sisfb_emi31,sisfb_emi32,sisfb_emi33;
+	CARD8 	sisfb_haveemilcd;
+
+	CARD8 	sisfb_lcdpdca;
+
+	CARD16  sisfb_tvxpos, sisfb_tvypos;	/* Warning: Values + 32 ! */
+
+	CARD32	sisfb_heapsize;			/* heap size (in KB) */
+	CARD32	sisfb_videooffset;		/* Offset of viewport in video memory (in bytes) */
+
+	CARD32	sisfb_curfstn;			/* currently running FSTN/DSTN mode */
+	CARD32	sisfb_curdstn;
+
+	CARD16	sisfb_pci_vendor;		/* PCI vendor (SiS or XGI) */
+
+	CARD8 reserved[190];			/* for future use */
+};
 
 /* Mandatory functions */
 static void SISIdentify(int flags);
@@ -1274,15 +1366,19 @@ static void SISLeaveVT(int scrnIndex, int flags);
 static Bool SISCloseScreen(int scrnIndex, ScreenPtr pScreen);
 static Bool SISSaveScreen(ScreenPtr pScreen, int mode);
 static Bool SISSwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
-static void SISAdjustFrame(int scrnIndex, int x, int y, int flags);
-#ifdef SISDUALHEAD
-static Bool SISSaveScreenDH(ScreenPtr pScreen, int mode);
-#endif
+void	    SISAdjustFrame(int scrnIndex, int x, int y, int flags);
 
 /* Optional functions */
+#ifdef SISDUALHEAD
+static Bool 	  SISSaveScreenDH(ScreenPtr pScreen, int mode);
+#endif
+#ifdef X_XF86MiscPassMessage
+extern int	  SISHandleMessage(int scrnIndex, const char *msgtype,
+				const char *msgval, char **retmsg);
+#endif
 static void       SISFreeScreen(int scrnIndex, int flags);
 static ModeStatus SISValidMode(int scrnIndex, DisplayModePtr mode,
-                               Bool verbose, int flags);
+				Bool verbose, int flags);
 
 /* Internally used functions */
 static Bool    SISMapMem(ScrnInfoPtr pScrn);
@@ -1293,66 +1389,110 @@ static Bool    SISUnmapIOPMem(ScrnInfoPtr pScrn);
 #endif
 static void    SISSave(ScrnInfoPtr pScrn);
 static void    SISRestore(ScrnInfoPtr pScrn);
-static void    SISVESARestore(ScrnInfoPtr pScrn);
 static Bool    SISModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
 static void    SISModifyModeInfo(DisplayModePtr mode);
 static void    SiSPreSetMode(ScrnInfoPtr pScrn, DisplayModePtr mode, int viewmode);
 static void    SiSPostSetMode(ScrnInfoPtr pScrn, SISRegPtr sisReg);
 static void    SiS6326PostSetMode(ScrnInfoPtr pScrn, SISRegPtr sisReg);
 static Bool    SiSSetVESAMode(ScrnInfoPtr pScrn, DisplayModePtr pMode);
+static void    SISVESARestore(ScrnInfoPtr pScrn);
 static void    SiSBuildVesaModeList(ScrnInfoPtr pScrn, vbeInfoPtr pVbe, VbeInfoBlock *vbe);
 static UShort  SiSCalcVESAModeIndex(ScrnInfoPtr pScrn, DisplayModePtr mode);
 static void    SISVESASaveRestore(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function);
 static void    SISBridgeRestore(ScrnInfoPtr pScrn);
 static void    SiSEnableTurboQueue(ScrnInfoPtr pScrn);
-unsigned char  SISSearchCRT1Rate(ScrnInfoPtr pScrn, DisplayModePtr mode);
+static void    SiSRestoreQueueMode(SISPtr pSiS, SISRegPtr sisReg);
+UChar  	       SISSearchCRT1Rate(ScrnInfoPtr pScrn, DisplayModePtr mode);
 static void    SISWaitVBRetrace(ScrnInfoPtr pScrn);
 void           SISWaitRetraceCRT1(ScrnInfoPtr pScrn);
 void           SISWaitRetraceCRT2(ScrnInfoPtr pScrn);
-static Bool    InRegion(int x, int y, region r);
-static USHORT  SiS_CheckModeCRT1(ScrnInfoPtr pScrn, DisplayModePtr mode,
-				 unsigned long VBFlags, BOOLEAN hcm);
-static USHORT  SiS_CheckModeCRT2(ScrnInfoPtr pScrn, DisplayModePtr mode,
-				 unsigned long VBFlags, BOOLEAN hcm);				 
+UShort         SiS_CheckModeCRT1(ScrnInfoPtr pScrn, DisplayModePtr mode,
+				 unsigned int VBFlags, Bool hcm);
+UShort         SiS_CheckModeCRT2(ScrnInfoPtr pScrn, DisplayModePtr mode,
+				 unsigned int VBFlags, Bool hcm);
+
 #ifdef SISMERGED
-static void    SISMergePointerMoved(int scrnIndex, int x, int y);
+static Bool    InRegion(int x, int y, region r);
+static void    SISMergedPointerMoved(int scrnIndex, int x, int y);
 #endif
-BOOLEAN        SiSBridgeIsInSlaveMode(ScrnInfoPtr pScrn);
-USHORT	       SiS_GetModeNumber(ScrnInfoPtr pScrn, DisplayModePtr mode, unsigned long VBFlags);
-unsigned char  SiS_GetSetBIOSScratch(ScrnInfoPtr pScrn, USHORT offset, unsigned char value);
+Bool           SiSBridgeIsInSlaveMode(ScrnInfoPtr pScrn);
+UShort	       SiS_GetModeNumber(ScrnInfoPtr pScrn, DisplayModePtr mode, unsigned int VBFlags);
+UChar  	       SiS_GetSetBIOSScratch(ScrnInfoPtr pScrn, UShort offset, UChar value);
 #ifdef DEBUG
 static void    SiSDumpModeInfo(ScrnInfoPtr pScrn, DisplayModePtr mode);
 #endif
-void           SISDetermineLCDACap(ScrnInfoPtr pScrn);
+Bool           SISDetermineLCDACap(ScrnInfoPtr pScrn);
 void           SISSaveDetectedDevices(ScrnInfoPtr pScrn);
+#ifdef SISGAMMARAMP
+void	       SISCalculateGammaRamp(ScreenPtr pScreen, ScrnInfoPtr pScrn);
+#endif
 
-extern USHORT   SiS_GetModeID(int VGAEngine, ULONG VBFlags, int HDisplay, int VDisplay,
-				  int Depth, BOOL FSTN, int LCDwith, int LCDheight);
-extern USHORT   SiS_GetModeID_LCD(int VGAEngine, ULONG VBFlags, int HDisplay, int VDisplay, int Depth,
-				  BOOLEAN FSTN, USHORT CustomT, int LCDwith, int LCDheight);
-extern USHORT   SiS_GetModeID_TV(int VGAEngine, ULONG VBFlags, int HDisplay, int VDisplay, int Depth);
-extern USHORT   SiS_GetModeID_VGA2(int VGAEngine, ULONG VBFlags, int HDisplay, int VDisplay, int Depth);
-extern int      SiSTranslateToVESA(ScrnInfoPtr pScrn, int modenumber);
-extern int      SiSTranslateToOldMode(int modenumber);
-extern BOOLEAN  SiSDetermineROMLayout661(SiS_Private *SiS_Pr, PSIS_HW_INFO HwInfo);
-extern BOOLEAN 	SiSBIOSSetMode(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension,
+/* Our very own vgaHW functions (sis_vga.c) */
+extern void 	SiSVGASave(ScrnInfoPtr pScrn, SISRegPtr save, int flags);
+extern void 	SiSVGARestore(ScrnInfoPtr pScrn, SISRegPtr restore, int flags);
+extern void 	SiSVGASaveFonts(ScrnInfoPtr pScrn);
+extern void 	SiSVGARestoreFonts(ScrnInfoPtr pScrn);
+extern void 	SISVGALock(SISPtr pSiS);
+extern void 	SiSVGAUnlock(SISPtr pSiS);
+extern void 	SiSVGAProtect(ScrnInfoPtr pScrn, Bool on);
+extern Bool 	SiSVGAMapMem(ScrnInfoPtr pScrn);
+extern void 	SiSVGAUnmapMem(ScrnInfoPtr pScrn);
+extern Bool 	SiSVGASaveScreen(ScreenPtr pScreen, int mode);
+
+/* shadow */
+extern void 	SISPointerMoved(int index, int x, int y);
+extern void 	SISPointerMovedReflect(int index, int x, int y);
+extern void 	SISRefreshArea(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
+extern void 	SISRefreshAreaReflect(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
+extern void 	SISRefreshArea8(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
+extern void 	SISRefreshArea16(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
+extern void 	SISRefreshArea24(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
+extern void 	SISRefreshArea32(ScrnInfoPtr pScrn, int num, BoxPtr pbox);
+
+/* vb */
+extern void 	SISCRT1PreInit(ScrnInfoPtr pScrn);
+extern void 	SISLCDPreInit(ScrnInfoPtr pScrn, Bool quiet);
+extern void 	SISTVPreInit(ScrnInfoPtr pScrn, Bool quiet);
+extern void 	SISCRT2PreInit(ScrnInfoPtr pScrn, Bool quiet);
+extern void 	SISSense30x(ScrnInfoPtr pScrn, Bool quiet);
+extern void 	SISSenseChrontel(ScrnInfoPtr pScrn, Bool quiet);
+extern void     SiSSetupPseudoPanel(ScrnInfoPtr pScrn);
+
+/* utility */
+extern void	SiSCtrlExtInit(ScrnInfoPtr pScrn);
+extern void	SiSCtrlExtUnregister(SISPtr pSiS, int index);
+
+/* init.c, init301.c ----- (use their data types!) */
+extern unsigned short	SiS_GetModeID(int VGAEngine, unsigned int VBFlags, int HDisplay, int VDisplay,
+				int Depth, BOOLEAN FSTN, int LCDwith, int LCDheight);
+extern unsigned short	SiS_GetModeID_LCD(int VGAEngine, unsigned int VBFlags, int HDisplay,
+				int VDisplay, int Depth, BOOLEAN FSTN, unsigned short CustomT,
+				int LCDwith, int LCDheight, unsigned int VBFlags2);
+extern unsigned short	SiS_GetModeID_TV(int VGAEngine, unsigned int VBFlags, int HDisplay,
+				int VDisplay, int Depth, unsigned int VBFlags2);
+extern unsigned short	SiS_GetModeID_VGA2(int VGAEngine, unsigned int VBFlags, int HDisplay,
+				int VDisplay, int Depth, unsigned int VBFlags2);
+extern int		SiSTranslateToVESA(ScrnInfoPtr pScrn, int modenumber);
+extern int		SiSTranslateToOldMode(int modenumber);
+extern BOOLEAN		SiSDetermineROMLayout661(struct SiS_Private *SiS_Pr);
+extern BOOLEAN		SiSBIOSSetMode(struct SiS_Private *SiS_Pr,
                                ScrnInfoPtr pScrn, DisplayModePtr mode, BOOLEAN IsCustom);
-extern BOOLEAN  SiSSetMode(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension,
-                           ScrnInfoPtr pScrn,USHORT ModeNo, BOOLEAN dosetpitch);
-extern void	SiSRegInit(SiS_Private *SiS_Pr, USHORT BaseAddr);
-extern void     SiSSetLVDSetc(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension,USHORT ModeNo);
-extern void     SiS_GetVBType(SiS_Private *SiS_Pr, PSIS_HW_INFO);
-extern DisplayModePtr SiSBuildBuiltInModeList(ScrnInfoPtr pScrn, BOOLEAN includelcdmodes,
-					       BOOLEAN isfordvi);
-extern BOOLEAN 	SiSBIOSSetModeCRT1(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension,
+extern BOOLEAN		SiSSetMode(struct SiS_Private *SiS_Pr,
+                           ScrnInfoPtr pScrn, unsigned short ModeNo, BOOLEAN dosetpitch);
+extern BOOLEAN		SiSBIOSSetModeCRT1(struct SiS_Private *SiS_Pr,
 				   ScrnInfoPtr pScrn, DisplayModePtr mode, BOOLEAN IsCustom);
-extern BOOLEAN 	SiSBIOSSetModeCRT2(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension,
+extern BOOLEAN		SiSBIOSSetModeCRT2(struct SiS_Private *SiS_Pr,
 				   ScrnInfoPtr pScrn, DisplayModePtr mode, BOOLEAN IsCustom);
+extern DisplayModePtr	SiSBuildBuiltInModeList(ScrnInfoPtr pScrn, BOOLEAN includelcdmodes,
+					      BOOLEAN isfordvi, BOOLEAN fakecrt2modes, BOOLEAN IsForCRT2);
+extern void		SiS_Chrontel701xBLOn(struct SiS_Private *SiS_Pr);
+extern void		SiS_Chrontel701xBLOff(struct SiS_Private *SiS_Pr);
+extern void		SiS_SiS30xBLOn(struct SiS_Private *SiS_Pr);
+extern void		SiS_SiS30xBLOff(struct SiS_Private *SiS_Pr);
+/* End of init.c, init301.c ----- */
 
-/* For power management for 315 series */
-extern void SiS_Chrontel701xBLOn(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension);
-extern void SiS_Chrontel701xBLOff(SiS_Private *SiS_Pr);
-extern void SiS_SiS30xBLOn(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension);
-extern void SiS_SiS30xBLOff(SiS_Private *SiS_Pr, PSIS_HW_INFO HwDeviceExtension);
+
+
+
 
 
